@@ -466,30 +466,32 @@ void STDCALLBULL FC_FUNC(matc,MATC) ( FC_CHAR_PTR(cmd,l1), FC_CHAR_PTR(Value,l2)
 #define MAXLEN 8192
 
   static int been_here = 0;
-    char *ptr, cc[32], *cmdcopy;
+  char *ptr, c, cc[32];
+  int slen = *len, i;
 
-    if ( been_here==0 ) {
-       mtc_init( NULL, stdout, stderr ); 
-       strcpy( cc, "format( 12,\"rowform\")" );
-       mtc_domath( cc );
-       been_here = 1;
-     }
+  if ( been_here==0 ) {
+     mtc_init( NULL, stdout, stderr ); 
+     strcpy( cc, "format( 12,\"rowform\")" );
+     mtc_domath( cc );
+     been_here = 1;
+   }
 
-    cmd[*len] = '\0';
-    if ( ptr = (char *)mtc_domath( cmd ) ) 
-    {
-      strcpy( Value, (char *)ptr );
-      *len = strlen(Value)-1;
+  c = cmd[slen];
+  cmd[slen] = '\0';
+  if ( ptr = (char *)mtc_domath(cmd) ) 
+  {
+    strcpy( Value, (char *)ptr );
+    *len = strlen(Value)-1; /* ignore linefeed! */
 
-      if ( strncmp( Value, "MATC ERROR:", 11 ) == 0 ) {
-          fprintf( stderr, "Solver input file error: %s\n", Value );
-          exit(0);
-      }
-    } else {
-      *len = 0;
-      *Value = ' ';
+    if ( strncmp( Value, "MATC ERROR:", 11 ) == 0 ) {
+        fprintf( stderr, "Solver input file error: %s\n", Value );
+        exit(0);
     }
-
+  } else {
+    *len = 0;
+    *Value = ' ';
+  }
+  cmd[slen]=c;
 }
 
 
@@ -531,3 +533,43 @@ void STDCALLBULL FC_FUNC(execsimulationproc,EXECSIMULATIONPROC)
 {
    DoSimulationProc( (void (STDCALLBULL *)())*Function,Model );
 }
+
+
+
+/*--------------------------------------------------------------------------
+  INTERNAL: execute (Krylov) iterator 
+  -------------------------------------------------------------------------*/
+static void DoIterCall( void (STDCALLBULL *iterProc)(),
+       void *x,void *b,void *ipar,void *dpar,void *work,
+       void (STDCALLBULL *mvProc)(),
+       void (STDCALLBULL *pcondProc)(),
+       void (STDCALLBULL *pcondrProc)(),
+       void (STDCALLBULL *dotProc)(),
+       void (STDCALLBULL *normProc)(),
+       void (STDCALLBULL *STOPC)() )
+/*
+       f_ptr mvProc, f_ptr pcondProc, f_ptr pcondrProc, f_ptr dotProc, f_ptr normProc, f_ptr STOPC )
+*/
+{ 
+  (*iterProc)( x,b,ipar,dpar,work,mvProc,pcondProc, 
+       pcondrProc,dotProc,normProc,STOPC );
+}
+
+/*--------------------------------------------------------------------------
+  This routine will call (Krylov) iterator
+  -------------------------------------------------------------------------*/
+void STDCALLBULL FC_FUNC(itercall,ITERCALL)
+     ( f_ptr iterProc, void *x, void *b, void *ipar, void *dpar, void *work, 
+       f_ptr mvProc, f_ptr pcondProc, f_ptr pcondrProc, f_ptr dotProc, f_ptr normProc, f_ptr STOPC )
+{
+   DoIterCall( (void (STDCALLBULL *)())*iterProc,x,b,ipar,dpar,work,
+       (void (STDCALLBULL *)())*mvProc, 
+       (void (STDCALLBULL *)())*pcondProc,
+       (void (STDCALLBULL *)())*pcondrProc,
+       (void (STDCALLBULL *)())*dotProc,
+       (void (STDCALLBULL *)())*normProc,
+       (void (STDCALLBULL *)())*STOPC );
+/*
+       mvProc, pcondProc, pcondrProc, dotProc, normProc, STOPC );
+*/
+}
