@@ -22,3 +22,9 @@
        PARAMETER (MPI_MAX=1)
        INTEGER MPI_SUM
        PARAMETER (MPI_SUM=1)
+       INTEGER MPI_LOGICAL
+       PARAMETER (MPI_LOGICAL=1)
+       INTEGER MPI_LAND
+       PARAMETER (MPI_LAND=1)
+       INTEGER MPI_LOR
+       PARAMETER (MPI_LOR=1)
